%% Supplementary materials for the paper
%% Eisma, Y. B., Hancock, P. A., & De Winter, J. C. F. (2020). On Senderss models of visual sampling behavior. Human Factors.

close all;clear all;clc %#ok<CLALL>
opengl software
rng(2); % rg should be 1, 2, 3, ... (this makes the random numbers repeatable)
flag=1;
% set to 1 for Figures 3, 5, 6, 7 (1000 sinusoids)
% set to 2 for Figures 2, 4 (2 sinusoids)
%%
% Figure 2. Two sinusoids (in gray) and the summed signal (in black). The summed signal has been divided by a constant so that the standard deviation equals 100 deg.
% Figure 3. Signal having a bandwidth of 0.48 Hz. The signal is the sum of 1,000 sinusoids. The summed signal has been divided by a constant so that the standard deviation equals 100 deg.

Fs=50;              % sampling frequency (Hz)
L=1000;             % length of the measurement (s)
t=(0:L*Fs)*(1/Fs);  % time vector

if flag==1
    cc=linspace(0.001,0.48,1000);
elseif flag==2
    cc=[0.03 0.48];
    yraw=NaN(length(cc),length(t));
end

% Construct the signal
d = 999999999*rand(length(cc),1); % random phase shifts
yt = zeros(1,length(t));
for i = 1:length(cc) % loop for making the signal
    sinusoid=360*sin(2*pi*cc(i)*t+d(i)); % individual sinusoid with random phase shift
    yt=yt+sinusoid; % add to total signal
    if flag==2;yraw(i,:)=sinusoid;end
end
%scaling_factor = 100/std(yt(1:60*Fs+1));
scaling_factor=100/(360*sqrt(length(cc))*sqrt(2)/2);
yt=yt*scaling_factor; % normalize the total signal based on the SD from 0 < t < = 60

if flag==1
    figure(3)
else
    figure(2)
end
hold on
if flag==2;plot(t,yraw','Linewidth',1,'color',[.5 .5 .5]);end % plot individual sinusoids
plot(t,yt,'k','Linewidth',5)
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',24)
set(gca,'LooseInset',[0.01 0.01 0.01 0.01],'ylim',[-360 360],'xlim',[0 60])
xlabel('Time (s)');ylabel('Signal (deg)')

if flag==1
    set(gcf,'WindowState', 'maximized');print(gcf,'Figure3.png','-dpng','-r300');
else
    set(gcf,'WindowState', 'maximized');print(gcf,'Figure2.png','-dpng','-r300');
end
%% create realizations starting at 70 deg
t2=(0:1.1*Fs)*(1/Fs);  % time vector
yraw2=NaN(100,length(t2));
count=0;
scaling_factor=100/(360*sqrt(length(cc))*sqrt(2)/2);
for reps=1:99999999999999
    d = 999999999*rand(length(cc),1); % random phase shifts
    yt = 0;
    for i = 1:length(cc) % loop for making the signal
        sinusoid=360*sin(d(i)); % individual sinusoid with random phase shift
        yt=yt+sinusoid; % add to total signal
    end
    if yt(1)*scaling_factor>69.9 && yt(1)*scaling_factor<71.1 % if the first sample is within range, then create the remaining few seconds
        yt = zeros(1,length(t2));
        for i2 = 1:length(cc) % loop for making the signal
            sinusoid=360*sin(2*pi*cc(i2)*t2+d(i2)); % individual sinusoid with random phase shift
            yt=yt+sinusoid; % add to total signal
        end
        count=count+1;
        disp(count)
        yraw2(count,:)=yt*scaling_factor;
        if count==size(yraw2,1)
            break
        end
    end
end

%% Figure 4. Illustration of the Nyquist-Shannon sampling theorem. If sampling at a frequency that is more than twice the bandwidth, signal reconstruction is possible. The gray dotted line represents the signal that is reconstructed from the samples. The samples are represented by the circular markers.
% code from Matth (2018) https://dsp.stackexchange.com/questions/37480/formulating-a-function-on-matlab-for-the-shannon-interpolation-formula
if flag==2
    F = 1000; % the sampling frequency used for drawing the continuous function
    Fs = [0.3 0.5 0.75 1]; % the sampling frequencies
    pos = [0.06 0.63 0.4 0.32;
        0.58 0.63 0.4 0.32
        0.06 0.1 0.4 0.32
        0.58 0.1 0.4 0.32];
    string='x=@(t) 180*(';
    for i=1:length(cc)
        string=[string ['+sin(2*pi*' num2str(cc(i)) '*t+' num2str(d(i)) ')']]; %#ok<AGROW>
    end
    string=[string ')*scaling_factor;'];
    eval(string)
    figure(4)
    for V=1:length(Fs) % loop over the different sampling frequencies
        t = -20:1/F:100; % time-samples for 'continuous' plotting
        ts = -20:1/Fs(V):100;  % time at the sample points
        xn = x(ts);  % discrete sequence
        interpolated = 0;
        for n=1:length(ts) % perform interpolation from xn to x:
            interpolated = interpolated + xn(n) * sinc((t-(n-1)/Fs(V))*Fs(V));
        end
        subplot(2,2,V);
        plot(t,x(t()),'k', 'linewidth',3); hold on; % plot continuous
        plot(ts,xn,'d','color',[.5 .5 .5],'markersize',12,'linewidth',3); % plot samples
        plot(t-20,interpolated,':','color',[.5 .5 .5],'Linewidth',3); % plot interpolated signal
        title(['Sampling rate = ' num2str(Fs(V)) ' Hz'])
        set(gca,'ylim',[-120 120],'xlim',[20 40],'pos',pos(V,:))
        box off
        if ismember(V,[3 4])
            xlabel('Time (s)');
        end
        if ismember(V,[1 3])
            ylabel('Pointer angle (deg)')
        end
    end
    fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',24)
    set(gca,'LooseInset',[0.01 0.01 0.01 0.01])
    set(gcf,'WindowState', 'maximized');print(gcf,'Figure4.png','-dpng','-r300');
end
%%
if flag==1
     % Figure 5. Senderss results show a convincingly strong correlation (r = .98 at individual and aggregate levels) between bandwidth and visual sampling. Five participants watched four dials and pressed a button when any of the four signals exceeded a threshold angle. Eye movements were recorded using a motion-picture camera.
    opengl hardware
    X = [0.5 1 2 4]/(2*pi);
    Y = [0.280195418	0.506966958	0.728681471	2.003305128
        0.277748178	0.493051103	0.642215339	1.635623143
        0.249853256	0.571362778	0.781500312	1.407325465
        0.133026902	0.500943677	0.515870937	1.584574262
        0.340852651	0.58297588	0.661468163	1.669622619];
    
    slope2 = [0	0
        1	2];
    
    p=polyfit(X,mean(Y),1);
    disp('Fit coefficients')
    disp(p)
    
    slope1 = [0 p(2)
        1 p(1)];
    
    c=corr(X',mean(Y)');
    disp('Figure 5. Correlation coefficient at the aggregate level')
    disp(c)
    disp('Figure 5. Mean correlation coefficient at the individual level')
    disp(mean(corr(X',Y')))
    figure(5);hold on;grid on
    plot(X,mean(Y),'ko','Markersize',20,'Linewidth',1,'markerfacecolor','k')
    plot(X,Y(1,:),'o','Markersize',10,'Linewidth',2,'markerfacecolor','none','markeredgecolor',[.7 .7 .7])
    plot(X,Y(2,:),'o','Markersize',10,'Linewidth',2,'markerfacecolor','none','markeredgecolor',[.7 .7 .7])
    plot(X,Y(3,:),'o','Markersize',10,'Linewidth',2,'markerfacecolor','none','markeredgecolor',[.7 .7 .7])
    plot(X,Y(4,:),'o','Markersize',10,'Linewidth',2,'markerfacecolor','none','markeredgecolor',[.7 .7 .7])
    plot(X,Y(5,:),'o','Markersize',10,'Linewidth',2,'markerfacecolor','none','markeredgecolor',[.7 .7 .7])
    plot(slope1(:,1)*2,slope1(:,2)*2,'k-','Linewidth',3)
    plot(slope2(:,1)*2,slope2(:,2)*2,'k--','Linewidth',3)
    text(0.14,1.2,'Best fit (0.05 + 2.44\it{W}\rm)')
    text(0.47,0.88,'Ideal observer (2\it{W}\rm)')
    xlabel('Signal bandwidth (\it{W}\rm) (Hz)')
    ylabel('Sampling frequency (Hz)')
    legend('Mean of 5 participants','Result of individual participant','location','northwest')
    axis square
    set(gca,'xlim',[0 0.8],'xtick',[0 0.08 0.16 0.32 0.64],'ylim',[0 2.2], 'LooseInset', [0.01 0.01 0.01 0.01])
    fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',24)
    set(gcf,'WindowState', 'maximized');print(gcf,'Figure5.png','-dpng','-r300');
    %% Figure 6. Example of 30 s of random sampling for a fixation duration of 0.3 s sampling probabilities of 6.7, 13.3, 26.7, and 53.3% for Dials 1, 2, 3, and 4, respectively.
    W=[0.08 0.16 0.32 0.64];
    p_i_cum=cumsum(W./sum(W));
    temp=rand(1000,1);
    dial_number(temp<p_i_cum(1))=1;
    dial_number(temp<p_i_cum(2)&temp>=p_i_cum(1))=2;
    dial_number(temp<p_i_cum(3)&temp>=p_i_cum(2))=3;
    dial_number(temp<p_i_cum(4)&temp>=p_i_cum(3))=4;
    [xb,yb]=stairs(dial_number);
    figure(6);plot(xb*0.3,yb,'k-o','linewidth',3,'markerfacecolor','k')
    fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',24)
    set(gca,'LooseInset',[0.01 0.01 0.01 0.01],'xlim',[0.5 30.5],'ytick',1:4,'yticklabel',{'1 (\it{W}\rm = 0.08 Hz)','2 (\it{W}\rm = 0.16 Hz)','3 (\it{W}\rm = 0.32 Hz)','4 (\it{W}\rm = 0.64 Hz)'});
    xlabel('Time (s)');ylabel('Dial number')
    set(gcf,'WindowState', 'maximized');print(gcf,'Figure6.png','-dpng','-r300');
    opengl software
    %% Figure 7. Left: Expected value and corresponding standard deviation for elapsed time (tau) since taking a reading of 70 deg. Right: the corresponding probability that the pointer angle exceeds 100 deg; the probability has a maximum at t + tau = 0.47 s.
    V=[10^-9 0.001:0.001:2];
    rho=sin(0.48*2*pi*V)./(0.48*2*pi*V);
    murt = 70*rho;
    sigma = 100*(1-rho.^2).^.5;
    
    PAT=NaN(1,length(V));
    for i = 1:length(V)
        PAT(i)=(1-normcdf(100,murt(i),sigma(i))); % percentage above threshold
    end
    
    figure(7);hold on;grid on
    subplot(1,2,1) % Mean and mean +/SD of pointer angle (deg)
    h1=plot(0.0:0.02:1,yraw2(:,1:51)','color',[0.8 0.8 0.8]);hold on;grid on;box off
    h2=plot(V,murt,'k','linewidth',3);hold on;grid on;box off
    h3=plot(V,murt-sigma,':','linewidth',3,'color','k');
    plot(V,murt+sigma,':','linewidth',3,'color','k');
    set(gca,'xlim',[0 1],'xtick',0:0.1:2,'pos',[0.06 0.1 0.4 0.87],'ylim',[-100 150])
    legend([h1(1) h2 h3],'Individual realizations','\mu',['\mu ' char(177) ' \sigma'],'location','southwest')
    ylabel('Pointer angle (deg)');xlabel('Elapsed time (s)')
    
    subplot(1,2,2) % Probability that pointer angle >100 deg
    plot(V,PAT,'k-','linewidth',3);hold on;grid on;box off
    [~,b]=max(PAT);
    plot([V(b) V(b)],[0 PAT(b)],'--','linewidth',3,'color',[.5 .5 .5]);
    
    set(gca,'xlim',[0 1],'xtick',0:0.1:2,'pos',[0.59 0.1 0.4 0.87])
    ylabel('Probability that pointer angle >100 deg');xlabel('Elapsed time (s)')
    fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',24)
    set(gcf,'WindowState', 'maximized');print(gcf,'Figure7.png','-dpng','-r300');
end
